//+------------------------------------------------------------------+
//|                                       MA - crosses histo MTF.mq4 |
//|                                         Copyright  2011, zznbrm |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, zznbrm"
#property indicator_separate_window
#property indicator_buffers   4
#property indicator_color1 Green
#property indicator_color2 Orange
#property indicator_color3 Orange
#property indicator_color4 Red
#property indicator_width1 2
#property indicator_width2 2
#property indicator_width3 2
#property indicator_width4 2
#property indicator_maximum 1
#property indicator_minimum 0
  
//+------------------------------------------------------------------+
//| User Inputs                                                      |
//+------------------------------------------------------------------+
extern int	  eintTF = PERIOD_M15;		
extern int    eintPeriod1 = 21;
extern int    eintPeriod2 = 34;
extern string estrMethod0 = "0=SMA,1=EMA,2=SMMA,3=LWMA";
extern int    eintMethod1 = MODE_SMA;
extern int    eintMethod2 = MODE_SMA;

//+------------------------------------------------------------------+
//| Buffers                                                          |
//+------------------------------------------------------------------+
double gadblUp[];
double gadblCrossUp[];
double gadblCrossDown[];
double gadblDown[];

//+------------------------------------------------------------------+
//| Global Variables                                                 |
//+------------------------------------------------------------------+

//*********************************************************************************************
//* STANDARD FUNCTIONS BELOW                                                                  *
//*********************************************************************************************
//+------------------------------------------------------------------+
//| init()                                                           |
//+------------------------------------------------------------------+
int init()
{
   string strShortName = "MA - crosses histo MTF (" + eintTF + ")";
   
   IndicatorShortName( strShortName );
   IndicatorDigits( 0 );
   
   SetIndexBuffer( 0, gadblUp );
   SetIndexLabel( 0, "Up" );
   SetIndexStyle( 0, DRAW_HISTOGRAM );
   SetIndexEmptyValue( 0, 0.0 );
   
   SetIndexBuffer( 1, gadblCrossUp );
   SetIndexLabel( 1, "CrossUp" );
   SetIndexStyle( 1, DRAW_HISTOGRAM );
   SetIndexEmptyValue( 1, 0.0 );
   
   SetIndexBuffer( 2, gadblCrossDown );
   SetIndexLabel( 2, "CrossDown" );
   SetIndexStyle( 2, DRAW_HISTOGRAM );
   SetIndexEmptyValue( 2, 0.0 );
   
   SetIndexBuffer( 3, gadblDown );
   SetIndexLabel( 3, "Down" );
   SetIndexStyle( 3, DRAW_HISTOGRAM );
   SetIndexEmptyValue( 3, 0.0 );
               
   return(0);
}

//+------------------------------------------------------------------+
//| deinit()                                                         |
//+------------------------------------------------------------------+
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//| start()                                                          |
//+------------------------------------------------------------------+
int start()
{  
   int intCountedBars = IndicatorCounted();

   if ( intCountedBars < 0 ) return( -1 );   
   if ( intCountedBars > 0 ) intCountedBars--;
   
   int jnx;
   int intLimit = Bars - intCountedBars;
   int intMax = Bars - ( MathMax( eintPeriod1, eintPeriod2 ) + 1 );
   double dblMA1, dblMA2;
   
   if ( intLimit > intMax )    intLimit = intMax;
            
   for( int inx = intLimit; inx >= 0; inx-- )
   {
      jnx = iBarShift( Symbol(), eintTF, Time[inx] );
      
      dblMA1 = calcMAAltClose( jnx, eintPeriod1, eintMethod1, Close[inx] ); 
      dblMA2 = calcMAAltClose( jnx, eintPeriod2, eintMethod2, Close[inx] );
      
      gadblUp[inx] = 0.0;
      gadblCrossUp[inx] = 0.0;
      gadblCrossDown[inx] = 0.0;
      gadblDown[inx] = 0.0;
            
      if ( dblMA1 > dblMA2 )
      {
         if ( ( gadblCrossUp[inx+1] > 0.0 ) || ( gadblUp[inx+1] > 0.0 ) )
            gadblUp[inx] = 1.0;
         else
            gadblCrossUp[inx] = 1.0;
      }
      else if ( dblMA1 < dblMA2 )
      {
         if ( ( gadblCrossDown[inx+1] > 0.0 ) || ( gadblDown[inx+1] > 0.0 ) )
            gadblDown[inx] = 1.0;
         else
            gadblCrossDown[inx] = 1.0;
      }
      else
      {
         gadblUp[inx] = gadblUp[inx+1];
         gadblCrossUp[inx] = gadblCrossUp[inx+1];
         gadblCrossDown[inx] = gadblCrossDown[inx+1];
         gadblDown[inx] = gadblDown[inx+1];
      }
                                      
   } 

   return(0);
}

//+------------------------------------------------------------------+
//| calcMAAltClose                                                   |
//+------------------------------------------------------------------+ 
double calcMAAltClose( int intShift, int intPeriod, int intMethod, double dblPrice )
{
   double adblPrice[], adblTemp[];
   
   ArrayResize( adblTemp, intPeriod+100 );
   
   ArraySetAsSeries( adblTemp, true );
   ArrayCopySeries( adblPrice, MODE_CLOSE, Symbol(), eintTF );
   ArrayCopy( adblTemp, adblPrice, 0, intShift, intPeriod+100 );
   
   adblTemp[0] = dblPrice;
   
   return( iMAOnArray( adblTemp, WHOLE_ARRAY, intPeriod, 0, intMethod, 0 ) );
}

